@@ -28,12 +28,10 @@
 /**** Init ****/
 
 value ocaml_av_init(value unit) {
-  caml_release_runtime_system();
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 9, 100)
   av_register_all();
 #endif
   avformat_network_init();
-  caml_acquire_runtime_system();
   return Val_unit;
 }
 
@@ -95,13 +93,7 @@ static void free_stream(stream_t *stream) {
 }
 
 static void close_av(av_t *av) {
-  if (!av)
-    return;
-
-  if (av->interrupt_cb != Val_none) {
-    caml_remove_generational_global_root(&av->interrupt_cb);
-    av->interrupt_cb = Val_none;
-  }
+  caml_release_runtime_system();
 
   if (av->format_context) {
     if (av->streams) {
@@ -130,24 +122,20 @@ static void close_av(av_t *av) {
     av->best_video_stream = NULL;
     av->best_subtitle_stream = NULL;
   }
-}
 
-static void free_av(av_t *av) {
-  if (!av)
-    return;
-
-  close_av(av);
+  caml_acquire_runtime_system();
 
   if (av->control_message_callback) {
-    caml_acquire_runtime_system();
     caml_remove_generational_global_root(&av->control_message_callback);
-    caml_release_runtime_system();
   }
 
-  free(av);
+  if (av->interrupt_cb != Val_none) {
+    caml_remove_generational_global_root(&av->interrupt_cb);
+    av->interrupt_cb = Val_none;
+  }
 }
 
-static void finalize_av(value v) { free_av(Av_val(v)); }
+static void finalize_av(value v) { free(Av_val(v)); }
 
 static struct custom_operations av_ops = {
     "ocaml_av_context",       finalize_av,
@@ -268,8 +256,6 @@ void ocaml_av_set_control_message_callback(value *p_av,
 
 typedef struct avio_t {
   AVFormatContext *format_context;
-  unsigned char *buffer;
-  int buffer_size;
   AVIOContext *avio_context;
   value read_cb;
   value write_cb;
@@ -455,11 +441,6 @@ static int ocaml_av_interrupt_callback(void *private) {
   av_t *av = (av_t *)private;
   int ret, n;
 
-  // This only happen when closing av_t so
-  // we want to return immediately.
-  if (av->interrupt_cb == Val_none)
-    return 1;
-
   ret = caml_c_thread_register();
 
   caml_acquire_runtime_system();
@@ -483,6 +464,8 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
   int (*write_cb)(void *opaque, uint8_t *buf, int buf_size) = NULL;
   int64_t (*seek_cb)(void *opaque, int64_t offset, int whence) = NULL;
   int write_flag = 0;
+  unsigned char *buffer;
+  int buffer_size;
 
   avio_t *avio = (avio_t *)calloc(1, sizeof(avio_t));
   if (!avio)
@@ -492,27 +475,22 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
   avio->write_cb = (value)NULL;
   avio->seek_cb = (value)NULL;
 
-  caml_release_runtime_system();
   avio->format_context = avformat_alloc_context();
 
   if (!avio->format_context) {
     free(avio);
-    caml_acquire_runtime_system();
     caml_raise_out_of_memory();
   }
 
-  avio->buffer_size = Int_val(bufsize);
-  avio->buffer = av_malloc(avio->buffer_size);
+  buffer_size = Int_val(bufsize);
+  buffer = av_malloc(buffer_size);
 
-  if (!avio->buffer) {
-    av_freep(avio->format_context);
-    caml_acquire_runtime_system();
+  if (!buffer) {
+    avformat_free_context(avio->format_context);
     free(avio);
     caml_raise_out_of_memory();
   }
 
-  caml_acquire_runtime_system();
-
   if (_read_cb != Val_none) {
     avio->read_cb = Some_val(_read_cb);
     caml_register_generational_global_root(&avio->read_cb);
@@ -532,18 +510,22 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
     seek_cb = ocaml_avio_seek_callback;
   }
 
-  caml_release_runtime_system();
   avio->avio_context =
-      avio_alloc_context(avio->buffer, avio->buffer_size, write_flag,
-                         (void *)avio, read_cb, write_cb, seek_cb);
-  caml_acquire_runtime_system();
+      avio_alloc_context(buffer, buffer_size, write_flag, (void *)avio, read_cb,
+                         write_cb, seek_cb);
 
   if (!avio->avio_context) {
-    caml_release_runtime_system();
-    av_freep(avio->buffer);
-    av_freep(avio->format_context);
-    caml_acquire_runtime_system();
+    if (avio->read_cb)
+      caml_remove_generational_global_root(&avio->read_cb);
 
+    if (avio->write_cb)
+      caml_remove_generational_global_root(&avio->write_cb);
+
+    if (avio->seek_cb)
+      caml_remove_generational_global_root(&avio->seek_cb);
+
+    av_freep(buffer);
+    avformat_free_context(avio->format_context);
     free(avio);
     caml_raise_out_of_memory();
   }
@@ -559,14 +541,11 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
 
 CAMLprim value caml_av_input_io_finalise(value _avio) {
   CAMLparam1(_avio);
-  // format_context and the buffer are freed as part of av_close.
   avio_t *avio = Avio_val(_avio);
 
-  caml_register_generational_global_root(&_avio);
-
-  caml_release_runtime_system();
-  av_freep(avio->avio_context);
-  caml_acquire_runtime_system();
+  // format_context is freed as part of close_av.
+  av_free(avio->avio_context->buffer);
+  avio_context_free(avio->avio_context);
 
   if (avio->read_cb)
     caml_remove_generational_global_root(&avio->read_cb);
@@ -579,8 +558,6 @@ CAMLprim value caml_av_input_io_finalise(value _avio) {
 
   free(avio);
 
-  caml_remove_generational_global_root(&_avio);
-
   CAMLreturn(Val_unit);
 }
 
@@ -747,9 +724,7 @@ CAMLprim value ocaml_av_open_input(value _url, value _format, value _interrupt,
     free(url);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -800,9 +775,7 @@ CAMLprim value ocaml_av_open_input_stream(value _avio, value _format,
   av_t *av = open_input(NULL, format, avio->format_context, Val_none, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -844,23 +817,14 @@ CAMLprim value ocaml_av_get_metadata(value _av, value _stream_index) {
 
   List_init(list);
 
-  caml_release_runtime_system();
-
   while ((tag = av_dict_get(metadata, "", tag, AV_DICT_IGNORE_SUFFIX))) {
-
-    caml_acquire_runtime_system();
-
     pair = caml_alloc_tuple(2);
     Store_field(pair, 0, caml_copy_string(tag->key));
     Store_field(pair, 1, caml_copy_string(tag->value));
 
     List_add(list, cons, pair);
-
-    caml_release_runtime_system();
   }
 
-  caml_acquire_runtime_system();
-
   CAMLreturn(list);
 }
 
@@ -925,16 +889,12 @@ static stream_t *allocate_stream_context(av_t *av, int index,
   if (!codec)
     return stream;
 
-  caml_release_runtime_system();
   stream->codec_context = avcodec_alloc_context3(codec);
 
   if (!stream->codec_context) {
-    caml_acquire_runtime_system();
     caml_raise_out_of_memory();
   }
 
-  caml_acquire_runtime_system();
-
   return stream;
 }
 
@@ -966,15 +926,14 @@ static stream_t *open_stream_index(av_t *av, int index, const AVCodec *dec) {
     caml_raise_out_of_memory();
 
   // initialize the stream parameters with demuxer information
-  caml_release_runtime_system();
   err = avcodec_parameters_to_context(stream->codec_context, dec_param);
 
   if (err < 0) {
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(err);
   }
 
   // Open the decoder
+  caml_release_runtime_system();
   err = avcodec_open2(stream->codec_context, dec, NULL);
   caml_acquire_runtime_system();
 
@@ -1031,8 +990,11 @@ static int decode_packet(av_t *av, stream_t *stream, AVPacket *packet,
   } else if (dec->codec_type == AVMEDIA_TYPE_SUBTITLE) {
     ret = avcodec_decode_subtitle2(dec, (AVSubtitle *)frame, &stream->got_frame,
                                    packet);
-    if (ret >= 0)
+    if (ret >= 0) {
       av_packet_unref(packet);
+      caml_acquire_runtime_system();
+      return ret;
+    }
   }
 
   av_packet_unref(packet);
@@ -1186,9 +1148,7 @@ CAMLprim value ocaml_av_read_input(value _packet, value _frame, value _av) {
       frame_kind = PVV_Subtitle_frame;
       frame_value = value_of_subtitle((AVSubtitle *)frame);
     } else {
-      caml_release_runtime_system();
       frame = av_frame_alloc();
-      caml_acquire_runtime_system();
 
       if (!frame) {
         av_packet_free(&packet);
@@ -1415,8 +1375,6 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
     av->interrupt_cb = Val_none;
   }
 
-  caml_release_runtime_system();
-
   ret = avformat_alloc_output_context2(&av->format_context, format, NULL,
                                        file_name);
 
@@ -1424,23 +1382,26 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
     if (file_name)
       free(file_name);
     av_dict_free(options);
-    free_av(av);
+    free(av);
 
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(ret);
   }
 
   ret = av_opt_set_dict(av->format_context, options);
 
   if (ret < 0) {
-    free_av(av);
+    if (av->interrupt_cb != Val_none) {
+      caml_remove_generational_global_root(&av->interrupt_cb);
+      av->interrupt_cb = Val_none;
+    }
+
+    free(av);
     if (file_name)
       free(file_name);
 
     av_dict_free(options);
-    free_av(av);
+    free(av);
 
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(ret);
   }
 
@@ -1448,12 +1409,11 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
     ret = av_opt_set_dict(av->format_context->priv_data, options);
 
     if (ret < 0) {
-      free_av(av);
+      free(av);
       if (file_name)
         free(file_name);
 
       av_dict_free(options);
-      caml_acquire_runtime_system();
       ocaml_avutil_raise_error(ret);
     }
   }
@@ -1461,13 +1421,12 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
   // open the output file, if needed
   if (avio_context) {
     if (av->format_context->oformat->flags & AVFMT_NOFILE) {
-      free_av(av);
+      free(av);
       if (file_name)
         free(file_name);
       av_dict_free(options);
 
       av_dict_free(options);
-      caml_acquire_runtime_system();
       Fail("Cannot set custom I/O on this format!");
     }
 
@@ -1475,17 +1434,23 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
     av->custom_io = 1;
   } else {
     if (!(av->format_context->oformat->flags & AVFMT_NOFILE)) {
+      caml_release_runtime_system();
       int err = avio_open2(&av->format_context->pb, file_name, AVIO_FLAG_WRITE,
                            interrupt_cb_ptr, options);
+      caml_acquire_runtime_system();
 
       if (err < 0) {
-        free_av(av);
+        if (av->interrupt_cb != Val_none) {
+          caml_remove_generational_global_root(&av->interrupt_cb);
+          av->interrupt_cb = Val_none;
+        }
+
+        free(av);
         if (file_name)
           free(file_name);
         av_dict_free(options);
 
         av_dict_free(options);
-        caml_acquire_runtime_system();
         ocaml_avutil_raise_error(err);
       }
 
@@ -1496,8 +1461,6 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
   if (file_name)
     free(file_name);
 
-  caml_acquire_runtime_system();
-
   return av;
 }
 
@@ -1531,9 +1494,7 @@ CAMLprim value ocaml_av_open_output(value _interrupt, value _format,
   av_t *av = open_output(format, filename, NULL, _interrupt, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -1580,9 +1541,7 @@ CAMLprim value ocaml_av_open_output_format(value _format, value _opts) {
   av_t *av = open_output(format, NULL, NULL, Val_none, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -1630,9 +1589,7 @@ CAMLprim value ocaml_av_open_output_stream(value _format, value _avio,
   av_t *av = open_output(format, NULL, avio->avio_context, Val_none, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -1733,13 +1690,10 @@ static void init_stream_encoder(AVBufferRef *device_ctx, AVBufferRef *frame_ctx,
   if (av->format_context->oformat->flags & AVFMT_GLOBALHEADER)
     enc_ctx->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
 
-  caml_release_runtime_system();
-
   if (device_ctx) {
     enc_ctx->hw_device_ctx = av_buffer_ref(device_ctx);
     if (!enc_ctx->hw_device_ctx) {
       av_dict_free(options);
-      caml_acquire_runtime_system();
       caml_raise_out_of_memory();
     }
   }
@@ -1748,16 +1702,16 @@ static void init_stream_encoder(AVBufferRef *device_ctx, AVBufferRef *frame_ctx,
     enc_ctx->hw_frames_ctx = av_buffer_ref(frame_ctx);
     if (!enc_ctx->hw_frames_ctx) {
       av_dict_free(options);
-      caml_acquire_runtime_system();
       caml_raise_out_of_memory();
     }
   }
 
+  caml_release_runtime_system();
   ret = avcodec_open2(enc_ctx, enc_ctx->codec, options);
+  caml_acquire_runtime_system();
 
   if (ret < 0) {
     av_dict_free(options);
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(ret);
   }
 
@@ -1768,21 +1722,23 @@ static void init_stream_encoder(AVBufferRef *device_ctx, AVBufferRef *frame_ctx,
 
   if (ret < 0) {
     av_dict_free(options);
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(ret);
   }
-
-  caml_acquire_runtime_system();
 }
 
 static stream_t *new_audio_stream(av_t *av, enum AVSampleFormat sample_fmt,
-                                  const AVCodec *codec,
+                                  int channels, const AVCodec *codec,
                                   AVDictionary **options) {
   stream_t *stream = new_stream(av, codec);
 
   AVCodecContext *enc_ctx = stream->codec_context;
 
   enc_ctx->sample_fmt = sample_fmt;
+  enc_ctx->channels = channels;
+// Detect new API
+#ifdef AV_CHANNEL_LAYOUT_MONO
+  av_channel_layout_default(&enc_ctx->ch_layout, channels);
+#endif
 
   init_stream_encoder(NULL, NULL, av, stream, options);
 
@@ -1808,7 +1764,8 @@ CAMLprim value ocaml_av_new_stream_copy(value _av, value _params) {
 }
 
 CAMLprim value ocaml_av_new_audio_stream(value _av, value _sample_fmt,
-                                         value _codec, value _opts) {
+                                         value _codec, value _channels,
+                                         value _opts) {
   CAMLparam2(_av, _opts);
   CAMLlocal2(ans, unused);
   const AVCodec *codec = AvCodec_val(_codec);
@@ -1829,13 +1786,11 @@ CAMLprim value ocaml_av_new_audio_stream(value _av, value _sample_fmt,
     }
   }
 
-  stream_t *stream =
-      new_audio_stream(Av_val(_av), Int_val(_sample_fmt), codec, &options);
+  stream_t *stream = new_audio_stream(Av_val(_av), Int_val(_sample_fmt),
+                                      Int_val(_channels), codec, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -1903,9 +1858,7 @@ CAMLprim value ocaml_av_new_video_stream(value _device_context,
       new_video_stream(device_ctx, frame_ctx, Av_val(_av), codec, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -1931,9 +1884,7 @@ static stream_t *new_subtitle_stream(av_t *av, const AVCodec *codec,
 
   int ret = subtitle_header_default(stream->codec_context);
   if (ret < 0) {
-    caml_release_runtime_system();
     av_dict_free(options);
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(ret);
   }
 
@@ -1967,9 +1918,7 @@ CAMLprim value ocaml_av_new_subtitle_stream(value _av, value _codec,
   stream_t *stream = new_subtitle_stream(Av_val(_av), codec, &options);
 
   // Return unused keys
-  caml_release_runtime_system();
   count = av_dict_count(options);
-  caml_acquire_runtime_system();
 
   unused = caml_alloc_tuple(count);
   AVDictionaryEntry *entry = NULL;
@@ -2212,21 +2161,20 @@ static void write_subtitle_frame(av_t *av, int stream_index,
   packet->data = NULL;
   packet->size = 0;
 
-  caml_release_runtime_system();
   err = av_new_packet(packet, size);
 
   if (err < 0) {
     av_packet_free(&packet);
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(err);
   }
 
+  caml_release_runtime_system();
   err = avcodec_encode_subtitle(stream->codec_context, packet->data,
                                 packet->size, subtitle);
+  caml_acquire_runtime_system();
 
   if (err < 0) {
     av_packet_free(&packet);
-    caml_acquire_runtime_system();
     ocaml_avutil_raise_error(err);
   }
 
@@ -2238,10 +2186,11 @@ static void write_subtitle_frame(av_t *av, int stream_index,
   packet->stream_index = stream_index;
   packet->pos = -1;
 
+  caml_release_runtime_system();
   err = av_interleaved_write_frame(av->format_context, packet);
+  caml_acquire_runtime_system();
 
   av_packet_free(&packet);
-  caml_acquire_runtime_system();
 
   if (err < 0)
     ocaml_avutil_raise_error(err);
@@ -2273,6 +2222,9 @@ CAMLprim value ocaml_av_flush(value _av) {
   av_t *av = Av_val(_av);
   int ret;
 
+  if (!av->header_written)
+    CAMLreturn(Val_unit);
+
   caml_release_runtime_system();
   ret = av_interleaved_write_frame(av->format_context, NULL);
   if (ret >= 0 && av->format_context->pb)
@@ -2338,17 +2290,23 @@ CAMLprim value ocaml_av_close(value _av) {
     }
 
     // write the trailer
-    caml_release_runtime_system();
-    if (av->header_written)
+    if (av->header_written) {
+      caml_release_runtime_system();
       av_write_trailer(av->format_context);
-    caml_acquire_runtime_system();
+      caml_acquire_runtime_system();
+    }
   }
 
-  caml_register_generational_global_root(&_av);
-  caml_release_runtime_system();
   close_av(av);
-  caml_acquire_runtime_system();
-  caml_remove_generational_global_root(&_av);
+
+  CAMLreturn(Val_unit);
+}
+
+CAMLprim value ocaml_av_cleanup_av(value _av) {
+  CAMLparam1(_av);
+  av_t *av = Av_val(_av);
+
+  close_av(av);
 
   CAMLreturn(Val_unit);
 }
@@ -2411,6 +2369,8 @@ CAMLprim value ocaml_av_codec_attr(value _stream) {
     } else {
       snprintf(attr, sizeof(attr), "avc1");
     }
+  } else if (stream->codecpar->codec_id == AV_CODEC_ID_FLAC) {
+    snprintf(attr, sizeof(attr), "fLaC");
   } else if (stream->codecpar->codec_id == AV_CODEC_ID_HEVC) {
     uint8_t *data = stream->codecpar->extradata;
     int profile = FF_PROFILE_UNKNOWN;
