@@ -1,5 +1,15 @@
 module C = Configurator.V1
 
+let has_bsf_h_code =
+  Printf.sprintf {|
+#include <libavcodec/bsf.h>
+
+int main()
+{
+  return 0;
+}
+|}
+
 let () =
   C.main ~name:"ffmpeg-avcodec-pkg-config" (fun c ->
       let default : C.Pkg_config.package_conf =
@@ -18,4 +28,11 @@ let () =
       in
       C.Flags.write_sexp "c_flags.sexp" conf.cflags;
       C.Flags.write_lines "c_flags" conf.cflags;
-      C.Flags.write_sexp "c_library_flags.sexp" conf.libs)
+      C.Flags.write_sexp "c_library_flags.sexp" conf.libs;
+
+      let has_bsf_h =
+        C.c_test ~c_flags:conf.cflags ~link_flags:conf.libs c has_bsf_h_code
+      in
+
+      C.C_define.gen_header_file c ~fname:"config.h"
+        [("HAS_BSF_H", Switch has_bsf_h)])
