@@ -10,9 +10,7 @@ type 'a container
 type audio = [ `Audio ]
 type video = [ `Video ]
 type subtitle = [ `Subtitle ]
-
-type media_type =
-  [ audio | video | subtitle | `Unknown | `Data | `Attachment | `Nb ]
+type media_type = Media_types.t
 
 (* Format *)
 type ('line, 'media) format
@@ -84,16 +82,7 @@ let () =
 
 let () =
   Callback.register_exception "ffmpeg_exn_error" (Error `Unknown);
-  Callback.register "ffmpeg_exn_failure" (fun s -> raise (Error (`Failure s)));
-  Callback.register "ffmpeg_gc_finalise" Gc.finalise
-
-external ocaml_avutil_register_lock_manager : unit -> bool
-  = "ocaml_avutil_register_lock_manager"
-  [@@noalloc]
-
-let () =
-  if not (ocaml_avutil_register_lock_manager ()) then
-    raise (Error (`Failure "Failed to register lock manager"))
+  Callback.register "ffmpeg_exn_failure" (fun s -> raise (Error (`Failure s)))
 
 type data =
   (int, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
@@ -115,7 +104,15 @@ end
 
 module Log = struct
   type level =
-    [ `Quiet | `Panic | `Fatal | `Error | `Warning | `Info | `Verbose | `Debug ]
+    [ `Quiet
+    | `Panic
+    | `Fatal
+    | `Error
+    | `Warning
+    | `Info
+    | `Verbose
+    | `Debug
+    | `Trace ]
 
   let int_of_level = function
     | `Quiet -> -8
@@ -126,6 +123,7 @@ module Log = struct
     | `Info -> 32
     | `Verbose -> 40
     | `Debug -> 48
+    | `Trace -> 56
 
   external set_level : int -> unit = "ocaml_avutil_set_log_level"
 
@@ -422,8 +420,14 @@ module Options = struct
     _cursor : _cursor option;
   }
 
-  external av_opt_next : _cursor option -> t -> _opt option
-    = "ocaml_avutil_av_opt_next"
+  exception Av_opt_iter_not_implemented of _cursor option
+
+  let () =
+    Callback.register_exception "av_opt_iter_not_implemented"
+      (Av_opt_iter_not_implemented None)
+
+  external av_opt_iter : _cursor option -> t -> _opt option
+    = "ocaml_avutil_av_opt_iter"
 
   let constant_of_opt opt (name, { _default; _ }) =
     let append fn l = (name, fn (Option.get _default)) :: l in
@@ -576,12 +580,13 @@ module Options = struct
     in
 
     let rec f _cursor _opts =
-      match av_opt_next _cursor v with
+      match av_opt_iter _cursor v with
         | None -> List.map opt_of_opt _opts
         | Some { _name; _spec = `Constant s; _cursor; _unit; _ } ->
             Hashtbl.add constants (Option.get _unit) (_name, s);
             f _cursor _opts
         | Some _opt -> f _opt._cursor (_opt :: _opts)
+        | exception Av_opt_iter_not_implemented _cursor -> f _cursor _opts
     in
 
     f None []
