@@ -14,9 +14,7 @@ type 'a container
 type audio = [ `Audio ]
 type video = [ `Video ]
 type subtitle = [ `Subtitle ]
-
-type media_type =
-  [ audio | video | subtitle | `Unknown | `Data | `Attachment | `Nb ]
+type media_type = Media_types.t
 
 (** {1 Format} *)
 
@@ -109,7 +107,15 @@ val time_base : unit -> rational
 
 module Log : sig
   type level =
-    [ `Quiet | `Panic | `Fatal | `Error | `Warning | `Info | `Verbose | `Debug ]
+    [ `Quiet
+    | `Panic
+    | `Fatal
+    | `Error
+    | `Warning
+    | `Info
+    | `Verbose
+    | `Debug
+    | `Trace ]
 
   val set_level : level -> unit
   val set_callback : (string -> unit) -> unit
